@@ -37,11 +37,11 @@ option_list = (
     make_option("--uid", action='store', dest='uid', default=None),
     make_option("--gid", action='store', dest='gid', default=None),
     make_option("--umask", action='store', dest='umask', default=0, type='int'),
-    make_option("--port", action='store', dest='port', default='0.0.0.0:8890',
-        help="Port for the sever, e.g. 0.0.0.0:8890 (default)"),
+    make_option("--listener", action='store', dest='listener', default='0.0.0.0:8890',
+        help="Listener address for the sever, e.g. 0.0.0.0:8890 (default)"),
     make_option("--detach", action='store_true', dest="detach", default=False,
         help="Detach process"),
-    make_option("--queue", action='store', dest='queue', default='memory',
+    make_option("--queue", action='store', dest='queue_type', default='memory',
         type='choice', choices=['memory', 'file', 'redis'],
         help="Queue type; memory=Memory queue (default), file=File based queue (persistent)"),
     make_option("-t", "--commit_timeout", action='store', dest='commit_timeout', default=1, type='int'),
