@@ -121,7 +121,7 @@ def with_retry(func):
                 continue
             except (IOError, RuntimeError, socket.error, ConnectionError):
                 exc_info = sys.exc_info()
-                time.sleep(delay)
+                self.sleep(delay)
                 retries += 1
                 delay *= 3  # growing the delay
 
@@ -135,8 +135,10 @@ class Connection(object):
 
     def __init__(self, host='localhost', port=8890, endpoints=None,
                  max_connect_retries=5, reconnect_delay=0.1,
-                 socket_timeout=4, encoding='utf-8',
-                 encoding_errors='strict'):
+                 socket_timeout=4, encoding='utf-8', encoding_errors='strict',
+                 socket_class=socket.socket, sleep=time.sleep):
+        self.socket_class = socket_class
+        self.sleep = sleep
         self.host = host
         self.port = port
         self.endpoints = endpoints
@@ -187,7 +189,7 @@ class Connection(object):
 
     def _connect(self):
         "Create a TCP socket connection"
-        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        sock = self.socket_class(socket.AF_INET, socket.SOCK_STREAM)
         sock.settimeout(self.socket_timeout)
 
         retries = 0
@@ -203,9 +205,9 @@ class Connection(object):
                     return sock   # we're good
                 if exc.errno == EINVAL:
                     # we're doomed, recreate socket
-                    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+                    sock = self.socket_class(socket.AF_INET, socket.SOCK_STREAM)
                     sock.settimeout(self.socket_timeout)
-                time.sleep(delay)
+                self.sleep(delay)
                 retries += 1
                 delay *= 3  # growing the delay
 
@@ -323,8 +325,10 @@ class ServerPool(object):
     def __init__(self, servers, max_pool_size=35, blacklist_time=60,
                  wait_for_connection=None, max_age=60, max_retries=3,
                  max_connect_retries=2, reconnect_delay=0.1,
-                 socket_timeout=4,
+                 socket_timeout=4, socket_class=socket.socket, sleep=time.sleep,
                  encoding='utf-8', encoding_errors='strict'):
+        self.socket_class = socket_class
+        self.sleep = sleep
         self.max_retries = max_retries
         self.max_connect_retries = max_connect_retries
         self.reconnect_delay = reconnect_delay
@@ -407,6 +411,8 @@ class ServerPool(object):
                 socket_timeout=self.socket_timeout,
                 encoding=self.encoding,
                 encoding_errors=self.encoding_errors,
+                socket_class=self.socket_class,
+                sleep=self.sleep,
             )
             connection.context = context
             try:
