@@ -6,7 +6,7 @@ import logging
 import xapian
 
 from . import json
-from .core import xapian_reopen, get_slot, expand_terms, find_terms, DOCUMENT_CUSTOM_TERM_PREFIX
+from .core import get_slot, get_prefix, expand_terms, find_terms, DOCUMENT_CUSTOM_TERM_PREFIX
 from .serialise import normalize, serialise_value
 from .exceptions import XapianError
 
@@ -14,21 +14,36 @@ MAX_DOCS = 10000
 
 
 class Search(object):
-    def __init__(self, database, obj,
+    def __init__(self, database, search,
                  get_matches=True, get_data=True, get_terms=False, get_size=False,
                  data='.', log=logging, dead=False):
-        self.log = log
-        self.data = data
+        self.database = database
+        self.search = search
+
         self.get_matches = get_matches
         self.get_terms = get_terms
         self.get_data = get_data
         self.get_size = get_size
-        self.size = None
+
+        self.data = data
+        self.log = log
         self.dead = dead
 
-        # SEARCH
+        self.spies = {}
+        self.warnings = []
+        self.produced = 0
+
+        self.size = None
+        self.facets = self.search.get('facets')
+        self.check_at_least = self.search.get('check_at_least', MAX_DOCS if self.facets else 0)
+        self.maxitems = self.search.get('maxitems', MAX_DOCS)
+        self.first = self.search.get('first', 0)
+
+        self.setup()
+
+    def setup(self):
         queryparser = xapian.QueryParser()
-        queryparser.set_database(database)
+        queryparser.set_database(self.database.database)
 
         query = None
 
@@ -37,7 +52,7 @@ class Search(object):
         def add_prefixes(string):
             for term, term_field, terms in find_terms(string):
                 if term_field and term_field not in prefixes:
-                    prefix = '%s%s' % (DOCUMENT_CUSTOM_TERM_PREFIX, get_slot(term_field))
+                    prefix = get_prefix(term_field, DOCUMENT_CUSTOM_TERM_PREFIX)
                     if term_field.lower() == term_field:
                         queryparser.add_prefix(term_field, prefix)
                     else:
@@ -45,57 +60,52 @@ class Search(object):
                     prefixes.add(term_field)
 
         # Build final query:
-        search = obj.get('search')
+        search = self.search.get('search')
         if search:
             if not isinstance(search, (tuple, list)):
                 search = [search]
             search = " AND ".join("(%s)" % s for s in search if s)
         if search and search != '(*)':
+            search = normalize(search).encode('utf-8')
+
+            ranges = self.search.get('ranges')
+            if ranges:
+                _ranges = set()
+                for field, begin, end in ranges:
+                    field = field.encode('utf-8')
+                    if field not in _ranges:
+                        slot = get_slot(field)
+                        vrp = xapian.StringValueRangeProcessor(slot, field)
+                        queryparser.add_valuerangeprocessor(vrp)
+                        _ranges.add(field)
+                    if begin is None:
+                        begin = b''
+                    if end is None:
+                        end = b''
+                    rng1 = b'(%s:%s..%s)' % (field, begin, end)
+                    rng2 = b'(%s:%s..%s)' % (field, serialise_value(begin)[0], serialise_value(end)[0])
+                    if rng1 == rng2:
+                        _search = search
+                        if rng1 in search:
+                            search = None
+                    else:
+                        _search = search.replace(rng1, rng2)
+                    if search != _search:
+                        search = _search
+                    else:
+                        search += b' AND %s' % rng2
+
+            search = expand_terms(search)
+            add_prefixes(search)
+            flags = xapian.QueryParser.FLAG_DEFAULT | xapian.QueryParser.FLAG_WILDCARD | xapian.QueryParser.FLAG_PURE_NOT
             try:
-                query = xapian.Query.unserialise(search)
-            except xapian.InvalidArgumentError:
-                pass
-            if not query:
-                search = normalize(search).encode('utf-8')
+                query = queryparser.parse_query(search, flags)
+            except (xapian.NetworkError, xapian.DatabaseModifiedError):
+                self.database.reopen()
+                queryparser.set_database(self.database.database)
+                query = queryparser.parse_query(search, flags)
 
-                ranges = obj.get('ranges')
-                if ranges:
-                    _ranges = set()
-                    for field, begin, end in ranges:
-                        field = field.encode('utf-8')
-                        if field not in _ranges:
-                            slot = get_slot(field)
-                            vrp = xapian.StringValueRangeProcessor(slot, field)
-                            queryparser.add_valuerangeprocessor(vrp)
-                            _ranges.add(field)
-                        if begin is None:
-                            begin = b''
-                        if end is None:
-                            end = b''
-                        rng1 = b'(%s:%s..%s)' % (field, begin, end)
-                        rng2 = b'(%s:%s..%s)' % (field, serialise_value(begin)[0], serialise_value(end)[0])
-                        if rng1 == rng2:
-                            _search = search
-                            if rng1 in search:
-                                search = None
-                        else:
-                            _search = search.replace(rng1, rng2)
-                        if search != _search:
-                            search = _search
-                        else:
-                            search += b' AND %s' % rng2
-
-                search = expand_terms(search)
-                add_prefixes(search)
-                flags = xapian.QueryParser.FLAG_DEFAULT | xapian.QueryParser.FLAG_WILDCARD | xapian.QueryParser.FLAG_PURE_NOT
-                try:
-                    query = queryparser.parse_query(search, flags)
-                except (xapian.NetworkError, xapian.DatabaseModifiedError):
-                    database = xapian_reopen(database, data=self.data, log=self.log)
-                    queryparser.set_database(database)
-                    query = queryparser.parse_query(search, flags)
-
-        partials = obj.get('partials')
+        partials = self.search.get('partials')
         if partials:
             if not isinstance(partials, (tuple, list)):
                 partials = [partials]
@@ -110,8 +120,8 @@ class Search(object):
                 try:
                     _partials_query = queryparser.parse_query(partial, flags)
                 except (xapian.NetworkError, xapian.DatabaseModifiedError):
-                    database = xapian_reopen(database, data=self.data, log=self.log)
-                    queryparser.set_database(database)
+                    self.database.reopen()
+                    queryparser.set_database(self.database.database)
                     _partials_query = queryparser.parse_query(partial, flags)
                 if partials_query:
                     partials_query = xapian.Query(
@@ -130,7 +140,7 @@ class Search(object):
             else:
                 query = partials_query
 
-        terms = obj.get('terms')
+        terms = self.search.get('terms')
         if terms:
             if not isinstance(terms, (tuple, list)):
                 terms = [terms]
@@ -142,8 +152,8 @@ class Search(object):
                 try:
                     terms_query = queryparser.parse_query(term, flags)
                 except (xapian.NetworkError, xapian.DatabaseModifiedError):
-                    database = xapian_reopen(database, data=self.data, log=self.log)
-                    queryparser.set_database(database)
+                    self.database.reopen()
+                    queryparser.set_database(self.database.database)
                     terms_query = queryparser.parse_query(term, flags)
                 if query:
                     query = xapian.Query(
@@ -160,18 +170,13 @@ class Search(object):
             else:
                 query = xapian.Query()
 
-        self.database = database
         self.query = query
-        self.facets = obj.get('facets')
-        self.check_at_least = obj.get('check_at_least', MAX_DOCS if self.facets else 0)
-        self.maxitems = obj.get('maxitems', MAX_DOCS)
-        self.first = obj.get('first', 0)
-        self.sort_by = obj.get('sort_by')
-        self.distinct = obj.get('distinct')
-        self.sort_by_reversed = obj.get('sort_by_reversed')
+        self.sort_by = self.search.get('sort_by')
+        self.distinct = self.search.get('distinct')
+        self.sort_by_reversed = self.search.get('sort_by_reversed')
 
-    def get_enquire(self, database):
-        enquire = xapian.Enquire(self.database)
+    def get_enquire(self):
+        enquire = xapian.Enquire(self.database.database)
         # enquire.set_weighting_scheme(xapian.BoolWeight())
         # enquire.set_docid_order(xapian.Enquire.DONT_CARE)
         # if weighting_scheme:
@@ -217,7 +222,7 @@ class Search(object):
 
         if self.distinct:
             if self.distinct is True:
-                field = 'id'
+                field = 'ID'
             else:
                 field = self.distinct
             enquire.set_collapse_key(get_slot(field))
@@ -227,11 +232,7 @@ class Search(object):
         return enquire
 
     def get_results(self):
-        try:
-            doccount = self.database.get_doccount()
-        except (xapian.NetworkError, xapian.DatabaseModifiedError):
-            self.database = xapian_reopen(self.database, data=self.data, log=self.log)
-            doccount = self.database.get_doccount()
+        doccount = self.database.get_doccount()
 
         maxitems = max(min(self.maxitems, doccount - self.first, MAX_DOCS), 0)
         check_at_least = max(min(self.check_at_least, doccount, MAX_DOCS), 0)
@@ -240,15 +241,15 @@ class Search(object):
             maxitems = 0
 
         try:
-            enquire = self.get_enquire(self.database)
+            enquire = self.get_enquire()
             matches = enquire.get_mset(self.first, maxitems, check_at_least)
         except (xapian.NetworkError, xapian.DatabaseModifiedError):
-            self.database = xapian_reopen(self.database, data=self.data, log=self.log)
+            self.database.reopen()
             try:
-                enquire = self.get_enquire(self.database)
+                enquire = self.get_enquire()
                 matches = enquire.get_mset(self.first, maxitems, check_at_least)
-            except (xapian.NetworkError, xapian.DatabaseError) as e:
-                raise XapianError(e)
+            except (xapian.NetworkError, xapian.DatabaseError) as exc:
+                raise XapianError(exc)
 
         self.produced = 0
         self.estimated = None
@@ -273,30 +274,23 @@ class Search(object):
 
         produced = 0
         for match in matches:
+            docid = match.docid
+            document = self.database.get_document(docid)
+
             self.dead or 'alive'  # Raises DeadException when needed
-            try:
-                id = match.document.get_value(get_slot('id'))
-            except (xapian.NetworkError, xapian.DatabaseModifiedError):
-                self.database = xapian_reopen(self.database, data=self.data, log=self.log)
-                id = match.document.get_value(get_slot('id'))
+            id = self.database.get_value(document, get_slot('ID'))
+
             produced += 1
             result = {
                 'id': id,
-                'docid': match.docid,
+                'docid': docid,
                 'rank': match.rank,
                 'weight': match.weight,
                 'percent': match.percent,
             }
             if self.get_data:
-                try:
-                    data = match.document.get_data()
-                except (xapian.NetworkError, xapian.DatabaseModifiedError):
-                    self.database = xapian_reopen(self.database, data=self.data, log=self.log)
-                    try:
-                        data = match.document.get_data()
-                    except xapian.NetworkError as e:
-                        raise XapianError(e)
-                except xapian.DocNotFoundError:
+                data = self.database.get_data(document)
+                if data is None:
                     continue
                 try:
                     data = json.loads(data)
@@ -307,7 +301,8 @@ class Search(object):
                 })
             if self.get_terms:
                 terms = []
-                for t in match.document.termlist():
+                termlist = self.database.get_termlist(document)
+                for t in termlist:
                     self.dead or 'alive'  # Raises DeadException when needed
                     terms.append(t.term.decode('utf-8'))
                 result.update({
