@@ -1,15 +1,16 @@
 from __future__ import unicode_literals, absolute_import
 
 import time
-import threading
 import logging
+import weakref
+import threading
+from hashlib import md5
 
 from functools import wraps
 
 from gevent import socket
 from gevent.server import StreamServer
 from gevent.threadpool import ThreadPool
-
 from ..utils import format_time
 
 
@@ -54,11 +55,11 @@ class AliveCommand(object):
         AliveCommand.cmds_duration += cmd_duration
         AliveCommand.cmds_count += 1
         logger(
-            "%s from %s %s%s ~%s (%0.3f cps)",
+            "%s %s%s by %s ~%s (%0.3f cps)",
             message % self.cmd_id,
-            self.origin,
             self.cmd,
             " -> %s" % results if results is not None else "",
+            self.origin,
             format_time(cmd_duration),
             AliveCommand.cmds_count / AliveCommand.cmds_duration,
         )
@@ -80,12 +81,17 @@ def command(threaded=False, **kwargs):
         func.threaded = threaded
         for attr, value in kwargs.items():
             setattr(func, attr, value)
-        if threaded:
+        if func.threaded:
             @wraps(func)
-            def wrapped(self, command, _sock, *args, **kwargs):
-                tid = threading.current_thread().name.rsplit('-', 1)[-1]
-                threading.current_thread().name = 'Command-%s-%s' % (command.cmd, tid)
-                client_socket = socket.socket(_sock=_sock)  # Create a gevent socket from the raw socket
+            def wrapped(self, command, client_socket, *args, **kwargs):
+                current_thread = threading.current_thread()
+                tid = current_thread.name.rsplit('-', 1)[-1]
+                current_thread.name = '%s-%s-%s' % (self.client_id[:14], command.cmd, tid)
+
+                # Create a gevent socket for this thread from the other tread's socket
+                # (using the raw underlying socket, '_sock'):
+                client_socket = socket.socket(_sock=client_socket._sock)
+
                 self.client_socket = client_socket
                 self.socket_file = client_socket.makefile()
                 try:
@@ -109,7 +115,7 @@ class ClientReceiver(object):
     def __init__(self, server, client_socket, address, log=logging,
                  encoding='utf-8', encoding_errors='strict'):
         self.log = log
-        self.server = server
+        self._server = weakref.ref(server)
         self.address = address
         self.local = threading.local()
         self.client_socket = client_socket
@@ -120,6 +126,15 @@ class ClientReceiver(object):
         self.cmd_id = 0
         self.activity = time.time()
 
+        self.client_id = ("Client-%s" % md5('%s:%s' % (address[0], address[1])).hexdigest())
+        current_thread = threading.current_thread()
+        tid = current_thread.name.rsplit('-', 1)[-1]
+        current_thread.name = '%s-%s' % (self.client_id[:14], tid)
+
+    @property
+    def server(self):
+        return self._server()
+
     def close(self):
         self.closed = True
 
@@ -159,7 +174,7 @@ class ClientReceiver(object):
             commands_pool = self.server.pool
             pool_size = self.server.pool_size
             pool_size_warning = self.server.pool_size_warning
-            commands_pool.spawn(func, command, self.client_socket._sock, line, command)
+            commands_pool.spawn(func, command, self.client_socket, line, command)
             pool_used = len(commands_pool)
             if not (pool_size_warning - pool_used) % 10:
                 self.log.warning("Commands pool is close to be full (%s/%s)", pool_used, pool_size)
@@ -173,10 +188,10 @@ class ClientReceiver(object):
             except DeadException:
                 command.cancelled()
 
-    def connectionMade(self):
+    def connectionMade(self, client):
         pass
 
-    def connectionLost(self):
+    def connectionLost(self, client):
         pass
 
     def sendLine(self, line):
@@ -195,49 +210,52 @@ class CommandServer(StreamServer):
     pool_size = 10
 
     def __init__(self, *args, **kwargs):
-        log = kwargs.pop('log', logging)
+        self.log = kwargs.pop('log', logging)
+        self.pool_size = kwargs.pop('pool_size', self.pool_size)
         super(CommandServer, self).__init__(*args, **kwargs)
-        self.log = log
         self.pool_size_warning = int(self.pool_size / 3.0 * 2.0)
         self.pool = ThreadPool(self.pool_size)
         self.clients = set()
 
-    def buildClient(self, client_socket, address):
+    def build_client(self, client_socket, address):
         return self.receiver_class(self, client_socket, address, log=self.log)
 
     def handle(self, client_socket, address):
-        client = self.buildClient(client_socket, address)
+        client = self.build_client(client_socket, address)
 
         self.clients.add(client)
-        client.connectionMade()
+        client.connectionMade(client)
         try:
             client.handle()
         finally:
             self.clients.discard(client)
-            client.connectionLost()
+            client.connectionLost(client)
 
-    def close(self, timeout=None):
+    def close(self, max_age=None):
         if self.closed:
-            if timeout is None:
+            if max_age is None:
                 self.log.error("Forcing server shutdown (%s clients)...", len(self.clients))
         else:
-            if timeout is None:
-                timeout = 10
+            if max_age is None:
+                max_age = 10
             self.log.warning("Hitting Ctrl+C again will terminate all running tasks!")
             super(CommandServer, self).close()
 
         now = time.time()
         clean = []
         for client in self.clients:
-            if timeout is None or client._weak or now - client.activity > timeout:
+            if max_age is None or client._weak or now - client.activity > max_age:
                 try:
                     client.client_socket._sock.close()
                 except AttributeError:
                     pass
                 clean.append(client)
+
         for client in clean:
             self.clients.discard(client)
 
+        return not bool(self.clients)
+
 
 class CommandReceiver(ClientReceiver):
     welcome = "# Welcome to the server! Type quit to exit."
@@ -246,13 +264,13 @@ class CommandReceiver(ClientReceiver):
         self._weak = False
         super(CommandReceiver, self).__init__(*args, **kwargs)
 
-    def connectionMade(self):
-        self.log.info("New connection from %s:%d (%d open connections)" % (self.address[0], self.address[1], len(self.server.clients)))
+    def connectionMade(self, client):
+        self.log.info("New connection from %s: %s:%d (%d open connections)" % (client.client_id, self.address[0], self.address[1], len(self.server.clients)))
         if self.welcome:
             self.sendLine(self.welcome)
 
-    def connectionLost(self):
-        self.log.info("Lost connection from %s:%d (%d open connections)" % (self.address[0], self.address[1], len(self.server.clients)))
+    def connectionLost(self, client):
+        self.log.info("Lost connection (%d open connections)" % len(self.server.clients))
 
     def lineReceived(self, line):
         super(CommandReceiver, self).lineReceived(line)
